import os
import shutil

from ajenti.api import *


@plugin
class ProxyFS (object):
    root = '/var/lib/elements/proxyfs'

    def init(self):
        if not os.path.exists(self.root):
            os.makedirs(self.root)

    def proxy_path(self, path):
        return self.root + path

    def exists(self, path):
        return os.path.exists(self.proxy_path(path))        

    def open(self, path, mode):
        path = self.proxy_path(path)
        dir = os.path.split(path)[0]
        if not os.path.exists(dir):
            os.makedirs(dir)
        return open(path, mode)

    def is_outdated(self, path):
        if not self.exists(path) or not os.path.exists(path):
            return True
        return os.stat(path).st_mtime > os.stat(self.proxy_path(path)).st_mtime

    def _vacuum_dir(self, path):
        if len(path) < 2:
            return
        if len(list(self.listdir(path))) == 0:
            shutil.rmtree(self.proxy_path(path))
            self._vacuum_dir(os.path.split(path)[0])

    def listdir(self, path):
        self._vacuum_dir(os.path.split(path)[0])
        for f in list(os.listdir(unicode(self.proxy_path(path)))):
            fpath = os.path.join(path, f)
            if not os.path.isdir(fpath) and self.is_outdated(fpath):
                self.remove(fpath)
                continue
            if not self.exists(fpath):
                continue
            yield f

    def remove(self, path):
        if self.exists(path):
            os.unlink(self.proxy_path(path))
