import os
import subprocess
import shutil
import tempfile

from ajenti.api import *
from ajenti.plugins import manager
from ajenti.plugins.tasks.api import Task

from proxyfs import ProxyFS


class MXFConvertTask (Task):
    ui = 'copydir'
    default_params = {
        'source': '',
        'destination': '',
    }
    toatom = True

    def run(self, source=None, destination=None):
        index = 0
        bindir = os.path.join(manager.resolve_path('elements'), 'bin')
        if not os.path.exists(destination):
            os.makedirs(destination)
        if not source.endswith('/'):
            source += '/'
        if not destination.endswith('/'):
            destination += '/'

        total = sum(len(x) for a,b,x in os.walk(source))

        for (dirpath, dirnames, filenames) in os.walk(source):
            for file in filenames:
                if file.lower().endswith('.mxf'):
                    srcfile = os.path.join(source, dirpath, file)
                    match_type = False
                    match_type = match_type or (self.toatom and  'MXF OP1a' in subprocess.check_output(['./mxfdump', '-dd=./dict.xml', srcfile], cwd=bindir))
                    match_type = match_type or (not self.toatom and  'MXF OP-Atom' in subprocess.check_output(['./mxfdump', '-dd=./dict.xml', srcfile], cwd=bindir))
                    if match_type:
                        destdir = destination + os.path.split(srcfile)[0][len(source):]
                        try:
                            os.makedirs(destdir)
                        except:
                            pass

                        tmpdir = tempfile.mkdtemp()
                        subprocess.call(['%s/mxfsplit' % bindir, '-dd=%s/dict.xml' % bindir, srcfile], cwd=tmpdir)
                        if self.toatom:
                            newname = file if len(os.listdir(tmpdir)) == 1 else None
                            for outfile in os.listdir(tmpdir):
                                subprocess.call(['%s/mxfwrap' % bindir, '-dp=%s' % bindir, '-a', os.path.join(tmpdir, outfile), os.path.join(destdir, newname or (outfile + '.mxf'))])
                        else:
                            subprocess.call(['%s/mxfwrap' % bindir, '-dp=%s' % bindir] + [os.path.join(tmpdir, outfile) for outfile in os.listdir(tmpdir)] + [os.path.join(destdir, file)])

                        shutil.rmtree(tmpdir)

                index += 1
                self.set_progress(index, total)


@plugin
class MXFConvertOP1AToAtomTask (MXFConvertTask):
    toatom = True
    name = 'Covert OP1A to OP-Atom'


@plugin
class MXFConvertAtomToOP1ATask (MXFConvertTask):
    toatom = False
    name = 'Covert OP-Atom to OP1A'


@plugin
class MakeProxyTask (Task):
    ui = 'elements:task-params-proxy'
    name = 'Update proxy files'
    default_params = {
        'source': '',
    }

    def run(self, source=None):
        index = 0
        if not source.endswith('/'):
            source += '/'

        total = sum(len(x) for a,b,x in os.walk(source))

        fs = ProxyFS.get()

        for (dirpath, dirnames, filenames) in os.walk(source):
            for file in filenames:
                srcfile = os.path.join(source, dirpath, file)
                if os.path.splitext(file.lower())[1] in ['.mxf', '.mov', '.m2v']:

                    if fs.exists(srcfile) and not fs.is_outdated(srcfile):
                        continue

                    fs.open(srcfile, 'w').close()
                    p = subprocess.Popen([
                        'ffmbc',
                        '-i', srcfile,
                        '-s', '1280x720',
                        '-vcodec', 'libx264',
                        '-preset', 'fast',
                        '-profile', 'baseline',
                        '-b', '1200k',
                        '-acodec', 'aac',
                        '-ac', '2', '-ar', '44100', '-ab', '128k',
                        '-y',
                        '-strict', 'experimental',
                        fs.proxy_path(srcfile)
                    ])

                    o,e = p.communicate()
                    if p.returncode:
                        print o, e

                index += 1
                self.message = srcfile
                self.set_progress(index, total)



@plugin
class CutClip (Task):
    ui = 'elements:task-cut'
    name = 'Cut video clip'
    default_params = {
        'source': '',
        'destination': '',
        'tfrom': 0,
        'tto': 0,
    }

    def run(self, source=None, destination=None, tfrom=None, tto=None):      
        p = subprocess.Popen([
            'ffmbc',
            '-i', source,
            '-ss', str(tfrom),
            '-t', str(tto - tfrom),
            '-vcodec', 'copy',
            '-acodec', 'copy',
            destination,
        ])
        print [
            'ffmbc',
            '-i', source,
            '-ss', str(tfrom),
            '-t', str(tto),
            '-vcodec', 'copy',
            '-acodec', 'copy',
            destination,
        ]
        o,e = p.communicate()
        if p.returncode:
            print o, e
        print o