import os

from ajenti.api import *
from ajenti.api.http import HttpPlugin, url
from ajenti.plugins.main.api import SectionPlugin
from ajenti.ui import on, p, UIElement
from ajenti.plugins import manager
from ajenti.plugins.tasks.manager import TaskManager

from ..tasks import CutClip
from ..proxyfs import ProxyFS


@plugin
class ElementsProjects (SectionPlugin):
    def init(self):
        self.title = 'Video'
        self.icon = 'suitcase'
        self.category = 'Elements'
        self.append(self.ui.inflate('elements:video-main'))

        self.opendialog = OpenProxyFileDialog.new(self.ui, visible=False)
        self.opendialog.on('select', self.on_open_dialog_select)
        self.append(self.opendialog)

        self.on_open_dialog_select(path=None)
        #self.on_open_dialog_select(path='/home/eugeny/sample.mov')
        self.cut_in_time = (None, 'not set')
        self.cut_out_time = (None, 'not set')

    def on_page_load(self):
        self.refresh()

    def refresh(self):
        self.find('cut-in').text = self.cut_in_time[1]
        self.find('cut-out').text = self.cut_out_time[1]
        if self.cut_in_time[0] and self.cut_out_time[0]:
            self.find('cut-save-box').visible = True
            self.find('cut-save-path').value = self.filepath + '-cut-%s-%s.mov' % (
                self.cut_in_time[1].replace(':', '_').replace('.', '_'),
                self.cut_out_time[1].replace(':', '_').replace('.', '_'),
            )
        else:
            self.find('cut-save-box').visible = False

    @on('player', 'cut-in')
    def on_cut_in(self, time=None, time_text=None):
        self.cut_in_time = (time, time_text)
        self.refresh()

    @on('player', 'cut-out')
    def on_cut_out(self, time=None, time_text=None):
        self.cut_out_time = (time, time_text)
        self.refresh()

    @on('cut-save', 'click')
    def on_cut_save(self):
        path = self.find('cut-save-path').value
        t = CutClip.new()
        t.params = {
            'source': self.filepath,
            'destination': path,
            'tfrom': self.cut_in_time[0],
            'tto': self.cut_out_time[0],
        }
        TaskManager.get(manager.context).run(task=t)
        self.context.notify('info', 'Cut is being saved')

    @on('open', 'click')
    def on_open(self):
        self.opendialog.visible = True

    def on_open_dialog_select(self, path=None):
        self.opendialog.visible = False
        self.filepath = path
        if path:
            self.find('player').path = path
            self.find('player').position = 0
            self.find('player').autoplay = True


@p('path', type=unicode)
@p('position', type=float)
@p('autoplay', type=bool)
@plugin
class VideoElement (UIElement):
    typeid = 'video'


from ajenti.plugins.main.controls_dialogs import OpenFileDialog

@plugin
class OpenProxyFileDialog (OpenFileDialog):
    def refresh(self):
        self._dirs = []
        self._files = []
        if len(self.path) > 1:
            self._dirs.append('..')
        for item in sorted(ProxyFS.get().listdir(self.path)):
            if os.path.isdir(os.path.join(self.path, item)):
                self._dirs.append(item)
            else:
                self._files.append(item)


@plugin
class ElementsVideoServer (HttpPlugin):
    def init(self):
        pass

    @url('/elements/video/(?P<path>.+)')
    def get_page(self, context, path):
        if context.session.identity is None:
            return context.respond_forbidden()

        return context.file(path, stream=True)
